@@ -41,12 +41,13 @@ int get_address()
 		mem_free(path);
 		return -1;
 	}
+	memset(su, 0, sizeof(struct sockaddr_un) + strlen(path) + 1);
 	su->sun_family = AF_UNIX;
 	add_to_strn(&path, LINKS_SOCK_NAME);
 	strcpy(su->sun_path, path);
 	mem_free(path);
 	s_unix = (struct sockaddr *)su;
-	s_unix_l = (char *)&su->sun_path - (char *)su + strlen(su->sun_path);
+	s_unix_l = (char *)&su->sun_path - (char *)su + strlen(su->sun_path) + 1;
 	return AF_UNIX;
 }
 
@@ -68,6 +69,7 @@ int get_address()
 		mem_free(sin);
 		return -1;
 	}
+	memset(sin, 0, sizeof(struct sockaddr_in));
 	sin->sin_family = AF_INET;
 	sin->sin_port = LINKS_PORT;
 	sin->sin_addr.s_addr = htonl(0x7f000001);
@@ -89,8 +91,8 @@ int bind_to_af_unix()
 	int cnt = 0;
 	int af;
 	if ((af = get_address()) == -1) return -1;
-	if ((s_unix_fd = socket(af, SOCK_STREAM, 0)) == -1) return -1;
 	again:
+	if ((s_unix_fd = socket(af, SOCK_STREAM, 0)) == -1) return -1;
 #if defined(SOL_SOCKET) && defined(SO_REUSEADDR)
 	setsockopt(s_unix_fd, SOL_SOCKET, SO_REUSEADDR, (void *)&a1, sizeof a1);
 #endif
@@ -138,8 +140,11 @@ int bind_to_af_unix()
 void af_unix_connection(void *xxx)
 {
 	int l = s_unix_l;
-	int ns = accept(s_unix_fd, (struct sockaddr *)s_unix_acc, &l);
+	int ns;
+	memset(s_unix_acc, 0, l);
+	ns = accept(s_unix_fd, (struct sockaddr *)s_unix_acc, &l);
 	init_term(ns, ns, win_func);
+	set_highpri();
 }
 
 void af_unix_close()
