@@ -197,7 +197,7 @@ void defrag_entry(struct cache_entry *e)
 void truncate_entry(struct cache_entry *e, int off, int final)
 {
 	struct fragment *f, *g;
-	if (e->length > off) e->length = off;
+	if (e->length > off) e->length = off, e->incomplete = 1;
 	foreach(f, e->frag) {
 		if (f->offset >= off) {
 			del:
