@@ -80,6 +80,8 @@ int get_pasv_socket(struct connection *c, int cc, int *sock, unsigned char *port
 	struct sockaddr_in sa;
 	struct sockaddr_in sb;
 	int len = sizeof(sa);
+	memset(&sa, 0, sizeof sa);
+	memset(&sb, 0, sizeof sb);
 	if (getsockname(cc, (struct sockaddr *)&sa, &len)) {
 		e:
 		setcstate(c, -errno);
@@ -197,7 +199,7 @@ void connected(struct connection *c)
 	setcstate(c, -errno);*/
 	if (getsockopt(*b->sock, SOL_SOCKET, SO_ERROR, (void *)&err, &len))
 		if (!(err = errno)) {
-			err = -S_STATE;
+			err = -(S_STATE);
 			goto bla;
 		}
 	if (err >= 10000) err -= 10000;	/* Why does EMX return so large values? */
