@@ -239,9 +239,9 @@ int is_path_prefix(unsigned char *d, unsigned char *s)
 	return !memcmp(d, s, dl);
 }
 
-int cookie_expired(struct cookie *c)
+int cookie_expired(struct cookie *c)	/* parse_http_date is broken */
 {
-  	return (c->expires && c->expires < time(NULL));
+  	return 0 && (c->expires && c->expires < time(NULL));
 }
 
 void send_cookies(unsigned char **s, int *l, unsigned char *url)
