@@ -782,6 +782,9 @@ Options are:\n\
  -dump\n\
   Write a plain-text version of the given HTML document to\n\
   stdout.\n\
+\n\
+ -width <size>\n\
+  Size of screen in characters, used in combination with -dump\n\
 \n\
  -no-connect\n\
   Runs links as a separate instance - instead of connecting to\n\
@@ -846,6 +849,8 @@ int max_tries = 3;
 int receive_timeout = 120;
 int unrestartable_receive_timeout = 600;
 
+int screen_width = 80;
+
 int max_format_cache_entries = 5;
 long memory_cache_size = 1048576;
 
@@ -870,7 +875,7 @@ unsigned char download_dir[MAX_STR_LEN] = "";
 unsigned char default_anon_pass[MAX_STR_LEN] = "somebody@host.domain";
 
 /* These are workarounds for some CGI script bugs */
-struct http_bugs http_bugs = { 0, 1, 0, 0 };
+struct http_bugs http_bugs = { 0, 1, 1, 0 };
 /*int bug_302_redirect = 0;*/
 	/* When got 301 or 302 from POST request, change it to GET
 	   - this violates RFC2068, but some buggy message board scripts rely on it */
@@ -898,6 +903,7 @@ struct option links_options[] = {
 	1, gen_cmd, num_rd, num_wr, 1, 16, &max_tries, "retries", "retries",
 	1, gen_cmd, num_rd, num_wr, 1, 1800, &receive_timeout, "receive_timeout", "receive-timeout",
 	1, gen_cmd, num_rd, num_wr, 1, 1800, &unrestartable_receive_timeout, "unrestartable_receive_timeout", "unrestartable-receive-timeout",
+	1, gen_cmd, num_rd, NULL, 10, 512, &screen_width, NULL, "width",
 	1, gen_cmd, num_rd, num_wr, 0, 256, &max_format_cache_entries, "format_cache_size", "format-cache-size",
 	1, gen_cmd, num_rd, num_wr, 0, MAXINT, &memory_cache_size, "memory_cache_size", "memory-cache-size",
 	1, gen_cmd, str_rd, str_wr, 0, MAX_STR_LEN, http_proxy, "http_proxy", "http-proxy",
