@@ -31,11 +31,13 @@ struct list_head dns_cache = {&dns_cache, &dns_cache};
 
 int do_real_lookup(unsigned char *name, ip *host)
 {
+	char *n;
 	struct hostent *hst;
+	for (n = name; *n; n++) if (*n != '.' && (*n < '0' || *n > '9')) goto nogethostbyaddr;
 #ifdef HAVE_GETHOSTBYADDR
 	if (!(hst = gethostbyaddr (name, strlen(name), AF_INET)))
 #endif
-		if (!(hst = gethostbyname(name)))
+		nogethostbyaddr: if (!(hst = gethostbyname(name)))
 			return -1;
 	memcpy(host, hst->h_addr_list[0], sizeof(ip));
 	return 0;
