@@ -1,5 +1,7 @@
 #include "links.h"
 
+int fast_ftp = 0;
+
 #define FTP_BUF	16384
 
 struct ftp_connection_info {
@@ -12,17 +14,19 @@ struct ftp_connection_info {
 	int dpos;
 	int buf_pos;
 	unsigned char ftp_buffer[FTP_BUF];
+	unsigned char cmdbuf[1];
 };
 
 void ftp_login(struct connection *);
 void ftp_logged(struct connection *);
+void ftp_sent_passwd(struct connection *);
 void ftp_got_reply(struct connection *, struct read_buffer *);
 void ftp_got_info(struct connection *, struct read_buffer *);
 void ftp_got_user_info(struct connection *, struct read_buffer *);
 void ftp_dummy_info(struct connection *, struct read_buffer *);
 void ftp_pass_info(struct connection *, struct read_buffer *);
-void ftp_send_retr_req(struct connection *);
-int add_file_cmd_to_str(unsigned char **, int *, struct connection *);
+void ftp_send_retr_req(struct connection *, int);
+struct ftp_connection_info *add_file_cmd_to_str(struct connection *);
 void ftp_retr_1(struct connection *);
 void ftp_retr_file(struct connection *, struct read_buffer *);
 void ftp_got_final_response(struct connection *, struct read_buffer *);
@@ -73,7 +77,7 @@ void ftp_func(struct connection *c)
 			return;
 		}
 		make_connection(c, p, &c->sock1, ftp_login);
-	} else ftp_send_retr_req(c);
+	} else ftp_send_retr_req(c, S_SENT);
 }
 
 void ftp_login(struct connection *c)
@@ -90,15 +94,19 @@ void ftp_login(struct connection *c)
 	if ((u = get_user_name(c->url)) && *u) add_to_str(&login, &logl, u);
 	else add_to_str(&login, &logl, "anonymous");
 	if (u) mem_free(u);
-	add_to_str(&login, &logl, "\r\nPASS ");
-	if ((u = get_pass(c->url)) && *u) add_to_str(&login, &logl, u);
-	else add_to_str(&login, &logl, default_anon_pass);
-	if (u) mem_free(u);
-	add_to_str(&login, &logl, "\r\n");
-	if (add_file_cmd_to_str(&login, &logl, c)) {
-		mem_free(login);
-		return;
-	}
+	if (fast_ftp) {
+		struct ftp_connection_info *fi;
+		add_to_str(&login, &logl, "\r\nPASS ");
+		if ((u = get_pass(c->url)) && *u) add_to_str(&login, &logl, u);
+		else add_to_str(&login, &logl, default_anon_pass);
+		if (u) mem_free(u);
+		add_to_str(&login, &logl, "\r\n");
+		if (!(fi = add_file_cmd_to_str(c))) {
+			mem_free(login);
+			return;
+		}
+		add_to_str(&login, &logl, fi->cmdbuf);
+	} else add_to_str(&login, &logl, "\r\n");
 	write_to_socket(c, c->sock1, login, strlen(login), ftp_logged);
 	mem_free(login);
 	setcstate(c, S_SENT);
@@ -127,8 +135,30 @@ void ftp_got_user_info(struct connection *c, struct read_buffer *rb)
 	if (!g) { read_from_socket(c, c->sock1, rb, ftp_got_user_info); return; }
 	if (g >= 530 && g < 540) { setcstate(c, S_FTP_LOGIN); retry_connection(c); return; }
 	if (g >= 400) { setcstate(c, S_FTP_UNAVAIL); retry_connection(c); return; }
-	if (g >= 200 && g < 300) ftp_dummy_info(c, rb);
-	else ftp_pass_info(c, rb);
+	if (g >= 200 && g < 300) {
+		if (fast_ftp) ftp_dummy_info(c, rb);
+		else ftp_send_retr_req(c, S_GETH);
+	} else {
+		if (fast_ftp) ftp_pass_info(c, rb);
+		else {
+			unsigned char *login;
+			unsigned char *u;
+			int logl = 0;
+			if (!(login = init_str())) {
+				setcstate(c, S_OUT_OF_MEM);
+				abort_connection(c);
+				return;
+			}
+			add_to_str(&login, &logl, "PASS ");
+			if ((u = get_pass(c->url)) && *u) add_to_str(&login, &logl, u);
+			else add_to_str(&login, &logl, default_anon_pass);
+			if (u) mem_free(u);
+			add_to_str(&login, &logl, "\r\n");
+			write_to_socket(c, c->sock1, login, strlen(login), ftp_sent_passwd);
+			mem_free(login);
+			setcstate(c, S_LOGIN);
+		}
+	}
 }
 
 void ftp_dummy_info(struct connection *c, struct read_buffer *rb)
@@ -139,89 +169,110 @@ void ftp_dummy_info(struct connection *c, struct read_buffer *rb)
 	ftp_retr_file(c, rb);
 }
 
+void ftp_sent_passwd(struct connection *c)
+{
+	struct read_buffer *rb;
+	if (!(rb = alloc_read_buffer(c))) return;
+	read_from_socket(c, c->sock1, rb, ftp_pass_info);
+}
+
 void ftp_pass_info(struct connection *c, struct read_buffer *rb)
 {
 	int g = get_ftp_response(c, rb, 0);
 	if (g == -1) { setcstate(c, S_FTP_ERROR); abort_connection(c); return; }
-	if (!g) { read_from_socket(c, c->sock1, rb, ftp_pass_info); return; }
+	if (!g) { read_from_socket(c, c->sock1, rb, ftp_pass_info); setcstate(c, S_LOGIN); return; }
 	if (g >= 530 && g < 540) { setcstate(c, S_FTP_LOGIN); abort_connection(c); return; }
 	if (g >= 400) { setcstate(c, S_FTP_UNAVAIL); abort_connection(c); return; }
-	ftp_retr_file(c, rb);
+	if (fast_ftp) ftp_retr_file(c, rb);
+	else ftp_send_retr_req(c, S_GETH);
 }
 
-int add_file_cmd_to_str(unsigned char **s, int *l, struct connection *c)
+struct ftp_connection_info *add_file_cmd_to_str(struct connection *c)
 {
 	unsigned char *d = get_url_data(c->url);
 	unsigned char *de;
 	unsigned char pc[6];
 	int ps;
-	struct ftp_connection_info *inf;
+	struct ftp_connection_info *inf, *inf2;
+	unsigned char *s;
+	int l;
 	if (!(inf = mem_alloc(sizeof(struct ftp_connection_info)))) {
+		oom:
 		setcstate(c, S_OUT_OF_MEM);
 		abort_connection(c);
-		return -1;
+		return NULL;
 	}
 	memset(inf, 0, sizeof(struct ftp_connection_info));
+	l = 0;
+	if (!(s = init_str())) {
+		mem_free(inf);
+		goto oom;
+	}
 	c->info = inf;
-	if ((ps = get_pasv_socket(c, c->sock1, &c->sock2, pc))) return ps;
+	if ((ps = get_pasv_socket(c, c->sock1, &c->sock2, pc))) return NULL;
 	if (!d) {
 		internal("get_url_data failed");
 		setcstate(c, S_INTERNAL);
 		abort_connection(c);
-		return -1;
+		return NULL;
 	}
 	if (!(de = strchr(d, POST_CHAR))) de = d + strlen(d);
 	if (d == de || de[-1] == '/') {
 		inf->dir = 1;
 		inf->pending_commands = 4;
-		add_to_str(s, l, "TYPE A\r\nPORT ");
-		add_num_to_str(s, l, pc[0]);
-		add_chr_to_str(s, l, ',');
-		add_num_to_str(s, l, pc[1]);
-		add_chr_to_str(s, l, ',');
-		add_num_to_str(s, l, pc[2]);
-		add_chr_to_str(s, l, ',');
-		add_num_to_str(s, l, pc[3]);
-		add_chr_to_str(s, l, ',');
-		add_num_to_str(s, l, pc[4]);
-		add_chr_to_str(s, l, ',');
-		add_num_to_str(s, l, pc[5]);
-		add_to_str(s, l, "\r\nCWD /");
-		add_bytes_to_str(s, l, d, de - d);
-		add_to_str(s, l, "\r\nLIST\r\n");
+		add_to_str(&s, &l, "TYPE A\r\nPORT ");
+		add_num_to_str(&s, &l, pc[0]);
+		add_chr_to_str(&s, &l, ',');
+		add_num_to_str(&s, &l, pc[1]);
+		add_chr_to_str(&s, &l, ',');
+		add_num_to_str(&s, &l, pc[2]);
+		add_chr_to_str(&s, &l, ',');
+		add_num_to_str(&s, &l, pc[3]);
+		add_chr_to_str(&s, &l, ',');
+		add_num_to_str(&s, &l, pc[4]);
+		add_chr_to_str(&s, &l, ',');
+		add_num_to_str(&s, &l, pc[5]);
+		add_to_str(&s, &l, "\r\nCWD /");
+		add_bytes_to_str(&s, &l, d, de - d);
+		add_to_str(&s, &l, "\r\nLIST\r\n");
 		c->from = 0;
 	} else {
 		inf->dir = 0;
 		inf->pending_commands = 3;
-		add_to_str(s, l, "TYPE I\r\nPORT ");
-		add_num_to_str(s, l, pc[0]);
-		add_chr_to_str(s, l, ',');
-		add_num_to_str(s, l, pc[1]);
-		add_chr_to_str(s, l, ',');
-		add_num_to_str(s, l, pc[2]);
-		add_chr_to_str(s, l, ',');
-		add_num_to_str(s, l, pc[3]);
-		add_chr_to_str(s, l, ',');
-		add_num_to_str(s, l, pc[4]);
-		add_chr_to_str(s, l, ',');
-		add_num_to_str(s, l, pc[5]);
+		add_to_str(&s, &l, "TYPE I\r\nPORT ");
+		add_num_to_str(&s, &l, pc[0]);
+		add_chr_to_str(&s, &l, ',');
+		add_num_to_str(&s, &l, pc[1]);
+		add_chr_to_str(&s, &l, ',');
+		add_num_to_str(&s, &l, pc[2]);
+		add_chr_to_str(&s, &l, ',');
+		add_num_to_str(&s, &l, pc[3]);
+		add_chr_to_str(&s, &l, ',');
+		add_num_to_str(&s, &l, pc[4]);
+		add_chr_to_str(&s, &l, ',');
+		add_num_to_str(&s, &l, pc[5]);
 		if (c->from) {
-			add_to_str(s, l, "\r\nREST ");
-			add_num_to_str(s, l, c->from);
+			add_to_str(&s, &l, "\r\nREST ");
+			add_num_to_str(&s, &l, c->from);
 			inf->rest_sent = 1;
 			inf->pending_commands++;
 		}
-		add_to_str(s, l, "\r\nRETR /");
-		add_bytes_to_str(s, l, d, de - d);
-		add_to_str(s, l, "\r\n");
+		add_to_str(&s, &l, "\r\nRETR /");
+		add_bytes_to_str(&s, &l, d, de - d);
+		add_to_str(&s, &l, "\r\n");
 	}
 	inf->opc = inf->pending_commands;
-	return 0;
+	if ((inf2 = mem_realloc(inf, sizeof(struct ftp_connection_info) + l + 1)))
+		strcpy((inf = inf2)->cmdbuf, s);
+	mem_free(s);
+	c->info = inf;
+	return inf;
 }
 
 
-void ftp_send_retr_req(struct connection *c)
+void ftp_send_retr_req(struct connection *c, int state)
 {
+	struct ftp_connection_info *fi;
 	unsigned char *login;
 	int logl = 0;
 	if (!(login = init_str())) {
@@ -229,13 +280,21 @@ void ftp_send_retr_req(struct connection *c)
 		abort_connection(c);
 		return;
 	}
-	if (add_file_cmd_to_str(&login, &logl, c)) {
+	if (!c->info && !(fi = add_file_cmd_to_str(c))) {
 		mem_free(login);
 		return;
+	} else fi = c->info;
+	if (fast_ftp) a:add_to_str(&login, &logl, fi->cmdbuf);
+	else {
+		unsigned char *nl = strchr(fi->cmdbuf, '\n');
+		if (!nl) goto a;
+		nl++;
+		add_bytes_to_str(&login, &logl, fi->cmdbuf, nl - fi->cmdbuf);
+		memmove(fi->cmdbuf, nl, strlen(nl) + 1);
 	}
 	write_to_socket(c, c->sock1, login, strlen(login), ftp_retr_1);
 	mem_free(login);
-	setcstate(c, S_SENT);
+	setcstate(c, state);
 }
 
 void ftp_retr_1(struct connection *c)
@@ -249,7 +308,13 @@ void ftp_retr_file(struct connection *c, struct read_buffer *rb)
 {
 	int g;
 	struct ftp_connection_info *inf = c->info;
-	rep:
+	if (0) {
+		rep:
+		if (!fast_ftp) {
+			ftp_send_retr_req(c, S_GETH);
+			return;
+		}
+	}
 	if (inf->pending_commands > 1) {
 		g = get_ftp_response(c, rb, 0);
 		if (g == -1) { setcstate(c, S_FTP_ERROR); abort_connection(c); return; }
