@@ -58,6 +58,8 @@ unsigned char *_xx_;
 	if ((_xx_ = s, l % ALLOC_GR) || (_xx_ = mem_realloc(s, l + ALLOC_GR))) s = _xx_, s[l++] = c;	\
 	} while (0)
 
+int get_attr_val_nl = 0;
+
 unsigned char *get_attr_val(unsigned char *e, unsigned char *name)
 {
 	unsigned char *n;
@@ -91,7 +93,7 @@ unsigned char *get_attr_val(unsigned char *e, unsigned char *name)
 			}
 			if (!f && *e != 13) {
 				if (*e != 9 && *e != 10) add_chr(a, l, *e);
-				else add_chr(a, l, ' ');
+				else if (!get_attr_val_nl) add_chr(a, l, ' ');
 			}
 			e++;
 		}
@@ -143,7 +145,16 @@ unsigned char *get_url_val(unsigned char *e, unsigned char *name)
 	return pp;
 }
 */
-#define get_url_val get_attr_val
+
+unsigned char *get_url_val(unsigned char *e, unsigned char *name)
+{
+	unsigned char *a;
+	get_attr_val_nl = 1;
+	a = get_attr_val(e, name);
+	get_attr_val_nl = 0;
+	return a;
+}
+
 
 struct {
 	int n;
