@@ -623,7 +623,7 @@ void line_break(struct part *p)
 	if (!p->data) goto e;
 	/*move_links(p, p->cx, p->cy, 0, p->cy + 1);*/
 	xpand_lines(p, p->cy + 1);
-	if (p->cx > par_format.leftmargin && (POS(p->cx-1, p->cy) & 0xff) == ' ') del_chars(p, p->cx-1, p->cy), p->cx--;
+	if (p->cx > par_format.leftmargin && LEN(p->cy) > p->cx - 1 && (POS(p->cx-1, p->cy) & 0xff) == ' ') del_chars(p, p->cx-1, p->cy), p->cx--;
 	/*if (LEN(p->cy) > p->x) p->x = LEN(p->cy);*/
 	if (p->cx > 0) align_line(p, p->cy);
 	if (p->data) for (t = last_tag_for_newline; t && (void *)t != &p->data->tags; t = t->prev) {
