@@ -187,7 +187,7 @@ void http_send_header(struct connection *c)
 	int http10 = http_bugs.http10;
 	struct cache_entry *e = NULL;
 	unsigned char *hdr;
-	unsigned char *h, *u;
+	unsigned char *h, *u, *uu;
 	int l = 0;
 	unsigned char *post;
 	unsigned char *host = upcase(c->url[0]) != 'P' ? c->url : get_url_data(c->url);
@@ -223,8 +223,21 @@ void http_send_header(struct connection *c)
 		http_end_request(c);
 		return;
 	}
-	if (!post) add_to_str(&hdr, &l, u);
-	else add_bytes_to_str(&hdr, &l, u, post - u - 1);
+	if (!post) uu = stracpy(u);
+	else uu = memacpy(u, post - u - 1);
+	while (strchr(uu, ' ')) {
+		unsigned char *sp = strchr(uu, ' ');
+		unsigned char *nu = mem_alloc(strlen(uu) + 3);
+		if (!nu) break;
+		memcpy(nu, uu, sp - uu);
+		nu[sp - uu] = 0;
+		strcat(nu, "%20");
+		strcat(nu, sp + 1);
+		mem_free(uu);
+		uu = nu;
+	}
+	add_to_str(&hdr, &l, uu);
+	mem_free(uu);
 	if (!http10) add_to_str(&hdr, &l, " HTTP/1.1\r\n");
 	else add_to_str(&hdr, &l, " HTTP/1.0\r\n");
 	if ((h = get_host_name(host))) {
@@ -240,6 +253,13 @@ void http_send_header(struct connection *c)
 	}
 	add_to_str(&hdr, &l, "User-Agent: Links (" VERSION_STRING "; ");
 	add_to_str(&hdr, &l, system_name);
+	if (!list_empty(terminals)) {
+		struct terminal *t = terminals.prev;
+		add_to_str(&hdr, &l, "; ");
+		add_num_to_str(&hdr, &l, t->x);
+		add_to_str(&hdr, &l, "x");
+		add_num_to_str(&hdr, &l, t->y);
+	}
 	add_to_str(&hdr, &l, ")\r\n");
 	add_to_str(&hdr, &l, "Accept: */*\r\n");
 	if (!(accept_charset)) {
@@ -581,15 +601,13 @@ void http_got_header(struct connection *c, struct read_buffer *rb)
 		unsigned char *ep;
 		int l = strtol(d, (char **)&ep, 10);
 		if (!*ep && l >= 0) {
-			if (!info->close) info->length = l;
+			if (!info->close || version >= 11) info->length = l;
 			c->est_length = c->from + l;
 		}
 		mem_free(d);
 	}
 	if ((d = parse_http_header(e->head, "Accept-Ranges", NULL))) {
-		if (!strcasecmp(d, "none")) {
-			if (!c->unrestartable) c->unrestartable = 1;
-		}
+		if (!strcasecmp(d, "none") && !c->unrestartable) c->unrestartable = 1;
 		mem_free(d);
 	} else if (!c->unrestartable && !c->from) c->unrestartable = 1;
 	if ((d = parse_http_header(e->head, "Transfer-Encoding", NULL))) {
