@@ -172,6 +172,7 @@ static int parse_act(unsigned char *s)
 		"page-up",
 		"paste-clipboard",
 		"previous-frame",
+		"really-quit",
 		"quit",
 		"reload",
 		"right",
@@ -328,7 +329,7 @@ static struct default_kb default_edit_keymap[] = {
 	{ ACT_END, KBD_END, 0 },
 	{ ACT_END, 'E', KBD_CTRL },
 	{ ACT_COPY_CLIPBOARD, KBD_INS, KBD_CTRL },
-	{ ACT_COPY_CLIPBOARD, 'B', KBD_CTRL },
+	{ ACT_COPY_CLIPBOARD, 'C', KBD_CTRL },
 	{ ACT_CUT_CLIPBOARD, 'X', KBD_CTRL },
 	{ ACT_PASTE_CLIPBOARD, 'V', KBD_CTRL },
 	{ ACT_ENTER, KBD_ENTER, 0 },
