@@ -444,20 +444,21 @@ struct xlist_head {
 
 #define init_list(x) {(x).next=&(x); (x).prev=&(x);}
 #define list_empty(x) ((x).next == &(x))
-#define del_from_list(x) {((struct list_head *)(x)->next)->prev=(x)->prev; ((struct list_head *)(x)->prev)->next=(x)->next;}
+#define del_from_list(x) {do_not_optimize_here(x); ((struct list_head *)(x)->next)->prev=(x)->prev; ((struct list_head *)(x)->prev)->next=(x)->next; do_not_optimize_here(x);}
 /*#define add_to_list(l,x) {(x)->next=(l).next; (x)->prev=(typeof(x))&(l); (l).next=(x); if ((l).prev==&(l)) (l).prev=(x);}*/
-#define add_at_pos(p,x) do {(x)->next=(p)->next; (x)->prev=(p); (p)->next=(x); (x)->next->prev=(x);} while(0)
 
 #ifdef HAVE_TYPEOF
+#define add_at_pos(p,x) do {do_not_optimize_here(p); (x)->next=(p)->next; (x)->prev=(p); (p)->next=(x); (x)->next->prev=(x);do_not_optimize_here(p);} while(0)
 #define add_to_list(l,x) add_at_pos((typeof(x))&(l),(x))
 #define foreach(e,l) for ((e)=(l).next; (e)!=(typeof(e))&(l); (e)=(e)->next)
 #define foreachback(e,l) for ((e)=(l).prev; (e)!=(typeof(e))&(l); (e)=(e)->prev)
 #else
-#define add_to_list(l,x) add_at_pos((struct xlist_head *)&(l),(struct xlist_head *)(x))
+#define add_at_pos(p,x) do {do_not_optimize_here(p); (x)->next=(p)->next; (x)->prev=(void *)(p); (p)->next=(x); (x)->next->prev=(x); do_not_optimize_here(p); } while(0)
+#define add_to_list(l,x) add_at_pos(&(l),(x))
 #define foreach(e,l) for ((e)=(l).next; (e)!=(void *)&(l); (e)=(e)->next)
 #define foreachback(e,l) for ((e)=(l).prev; (e)!=(void *)&(l); (e)=(e)->prev)
 #endif
-#define free_list(l) {while ((l).next != &(l)) {struct list_head *a=(l).next; del_from_list(a); mem_free(a); }}
+#define free_list(l) {do_not_optimize_here(&l); while ((l).next != &(l)) {struct list_head *a__=(l).next; del_from_list(a__); mem_free(a__); } do_not_optimize_here(&l);}
 
 #define WHITECHAR(x) ((x) == 9 || (x) == 10 || (x) == 12 || (x) == 13 || (x) == ' ')
 #define U(x) ((x) == '"' || (x) == '\'')
@@ -545,6 +546,12 @@ int can_resize_window(int);
 int can_open_os_shell(int);
 struct open_in_new *get_open_in_new(int);
 void set_highpri();
+#ifdef HAVE_OPEN_PREALLOC
+int open_prealloc(char *, int, int, int);
+void prealloc_truncate(int, int);
+#else
+static inline void prealloc_truncate(int x, int y) {}
+#endif
 
 /* select.c */
 
@@ -965,9 +972,9 @@ struct window {
 	struct terminal *term;
 };
 
-#define MAX_TERM_LEN	16	/* this must be multiple of 8! (alignment problems) */
+#define MAX_TERM_LEN	32	/* this must be multiple of 8! (alignment problems) */
 
-#define MAX_CWD_LEN	8192	/* this must be multiple of 8! (alignment problems) */	
+#define MAX_CWD_LEN	256	/* this must be multiple of 8! (alignment problems) */	
 
 #define ENV_XWIN	1
 #define ENV_SCREEN	2
@@ -2006,6 +2013,7 @@ extern int download_utime;
 extern int max_connections;
 extern int max_connections_to_host;
 extern int max_tries;
+extern int screen_width;
 extern int receive_timeout;
 extern int unrestartable_receive_timeout;
 
