@@ -170,6 +170,7 @@ void end_dump(struct status *stat, void *p)
 	if (dmp == D_SOURCE) {
 		if (ce) {
 			struct fragment *frag;
+			nextfrag:
 			foreach(frag, ce->frag) if (frag->offset <= dump_pos && frag->offset + frag->length > dump_pos) {
 				int l = frag->length - (dump_pos - frag->offset);
 				int w = hard_write(oh, frag->data + dump_pos - frag->offset, l);
@@ -182,6 +183,7 @@ void end_dump(struct status *stat, void *p)
 				}
 				dump_pos += w;
 				detach_connection(stat, dump_pos);
+				goto nextfrag;
 			}
 		}
 		if (stat->state >= 0) return;
@@ -196,7 +198,7 @@ void end_dump(struct status *stat, void *p)
 		memset(&fd, 0, sizeof(struct f_data_c));
 		o.xp = 0;
 		o.yp = 1;
-		o.xw = 80;
+		o.xw = screen_width;
 		o.yw = 25;
 		o.col = 0;
 		o.cp = 0;
