@@ -207,9 +207,9 @@ void memory_info(struct terminal *term, void *d, struct session *ses)
 	r->data = d;
 	r->timer = -1;
 	p = message;
-	p += sprintf(p, "%ld %s", mem_amount, _(TEXT(T_MEMORY_ALLOCATED), term));
-	if (last_mem_amount != -1) p += sprintf(p, ", %s %ld, %s %ld", _(TEXT(T_LAST), term), last_mem_amount, _(TEXT(T_DIFFERENCE), term), mem_amount - last_mem_amount);
-	p += sprintf(p, ".");
+	sprintf(p, "%ld %s", mem_amount, _(TEXT(T_MEMORY_ALLOCATED), term)), p += strlen(p);
+	if (last_mem_amount != -1) sprintf(p, ", %s %ld, %s %ld", _(TEXT(T_LAST), term), last_mem_amount, _(TEXT(T_DIFFERENCE), term), mem_amount - last_mem_amount), p += strlen(p);
+	sprintf(p, "."), p += strlen(p);
 #if 0 && defined(MAX_LIST_SIZE)
 	if (last_mem_amount != -1) {
 		long i, j;
@@ -305,8 +305,11 @@ void downloads_menu(struct terminal *term, void *ddd, struct session *ses)
 	struct menu_item *mi = NULL;
 	int n = 0;
 	foreachback(d, downloads) {
+		unsigned char *u;
 		if (!mi) if (!(mi = new_menu(3))) return;
-		add_to_menu(&mi, stracpy(d->url), "", "", MENU_FUNC display_download, d, 0);
+		u = stracpy(d->url);
+		if (strchr(u, POST_CHAR)) *strchr(u, POST_CHAR) = 0;
+		add_to_menu(&mi, u, "", "", MENU_FUNC display_download, d, 0);
 		n++;
 	}
 	if (!n) do_menu(term, no_downloads_menu, ses);
@@ -987,8 +990,8 @@ struct menu_item file_menu22[] = {
 #endif
 #ifdef LEAK_DEBUG
 	TEXT(T_MEMORY_INFO), "", TEXT(T_HK_MEMORY_INFO), MENU_FUNC memory_info, (void *)0, 0, 0,
-	"", "", M_BAR, NULL, NULL, 0, 0,
 #endif
+	"", "", M_BAR, NULL, NULL, 0, 0,
 };
 
 struct menu_item file_menu3[] = {
