@@ -57,6 +57,20 @@ int get_e(char *env)
 
 #endif
 
+#if defined(O_SIZE) && defined(__EMX__)
+
+int open_prealloc(char *name, int flags, int mode, int siz)
+{
+	return open(name, flags | O_SIZE, mode, (unsigned long)siz);
+}
+
+void prealloc_truncate(int h, int siz)
+{
+	ftruncate(h, siz);
+}
+
+#endif
+
 /* Terminal size */
 
 #if defined(UNIX) || defined(BEOS) || defined(RISCOS) || defined(WIN32)
@@ -1079,6 +1093,8 @@ int start_thread(void (*fn)(void *, int), void *ptr, int l)
 	fcntl(p[0], F_SETFL, O_NONBLOCK);
 	fcntl(p[1], F_SETFL, O_NONBLOCK);
 	if (!(f = fork())) {
+		struct terminal *t;
+		foreach (t, terminals) if (t->fdin > 0) close(t->fdin);
 		close(p[0]);
 		fn(ptr, p[1]);
 		write(p[1], "x", 1);
