@@ -49,3 +49,9 @@ void cfmakeraw(struct termios *t);
 #endif
 #define errno 1
 #endif
+
+#if defined(O_SIZE) && defined(__EMX__)
+#define HAVE_OPEN_PREALLOC
+#endif
+
+
