@@ -287,8 +287,11 @@ int check_signals()
 
 void sigchld(void *p)
 {
-	/*wait(NULL);*/
-	waitpid(-1, NULL, WNOHANG);
+#ifndef WNOHANG
+	wait(NULL);
+#else
+	while ((int)waitpid(-1, NULL, WNOHANG) > 0) ;
+#endif
 }
 
 void set_sigcld()
