@@ -306,7 +306,7 @@ void abort_download(struct download *down)
 	if (down->ask) delete_window(down->ask);
 	if (down->stat.state >= 0) change_connection(&down->stat, NULL, PRI_CANCEL);
 	mem_free(down->url);
-	if (down->handle != -1) close(down->handle);
+	if (down->handle != -1) prealloc_truncate(down->handle, down->last_pos), close(down->handle);
 	if (down->prog) {
 		unlink(down->file);
 		mem_free(down->prog);
@@ -467,7 +467,7 @@ void display_download(struct terminal *term, struct download *down, struct sessi
 	do_dialog(term, dlg, getml(dlg, NULL));
 }
 
-time_t parse_http_date(const char *date)
+time_t parse_http_date(const char *date)	/* this functions is bad !!! */
 {
 	const char *months[12] =
 		{"Jan", "Feb", "Mar", "Apr", "May", "Jun",
@@ -529,8 +529,20 @@ void download_data(struct status *stat, struct download *down)
 		}*/
 	}
 	foreach(frag, ce->frag) if (frag->offset <= down->last_pos && frag->offset + frag->length > down->last_pos) {
-		int w = write(down->handle, frag->data + down->last_pos - frag->offset, frag->length - (down->last_pos - frag->offset));
+		int w;
+#ifdef HAVE_OPEN_PREALLOC
+		if (!down->last_pos && (!down->stat.prg || down->stat.prg->size > 0)) {
+			close(down->handle);
+			down->handle = open_prealloc(down->file, O_CREAT|O_WRONLY|O_TRUNC, 0666, down->stat.prg ? down->stat.prg->size : ce->length);
+			if (down->handle == -1) goto error;
+			set_bin(down->handle);
+		}
+#endif
+		w = write(down->handle, frag->data + down->last_pos - frag->offset, frag->length - (down->last_pos - frag->offset));
 		if (w == -1) {
+#ifdef HAVE_OPEN_PREALLOC
+			error:
+#endif
 			detach_connection(stat, down->last_pos);
 			if (!list_empty(sessions)) {
 				unsigned char *msg = stracpy(down->file);
@@ -554,6 +566,7 @@ void download_data(struct status *stat, struct download *down)
 			}
 		} else {
 			if (down->prog) {
+				prealloc_truncate(down->handle, down->last_pos);
 				close(down->handle), down->handle = -1;
 				exec_on_terminal(get_download_ses(down)->term, down->prog, down->file, !!down->prog_flags);
 				mem_free(down->prog), down->prog = NULL;
@@ -1572,7 +1585,7 @@ unsigned char *get_current_url(struct session *ses, unsigned char *str, size_t s
 	}
 
 	/* Ensure that the url size is not greater than str_size */ 
-	if (url_len > str_size)
+	if (url_len >= str_size)
 			url_len = str_size - 1;
 
 	strncpy(str, here, url_len);
