@@ -73,7 +73,7 @@ unsigned char *get_content_type(unsigned char *head, unsigned char *url)
 	add_bytes_to_str(&exxt, &el, ext, extl);
 	foreach(a, assoc) if (is_in_list(a->ct, exxt, el)) return exxt;
 	mem_free(exxt);
-	return stracpy("text/plain");
+	return !force_html ? stracpy("text/plain") : stracpy("text/html");
 }
 
 struct assoc *get_type_assoc(struct terminal *term, unsigned char *type)
