@@ -383,7 +383,10 @@ void clear_link(struct terminal *t, struct f_data_c *scr)
 	}
 }
 
-inline int srch_cmp(unsigned char c1, unsigned char c2)
+#ifdef __GNUC__
+inline			/* solaris CC bug */
+#endif
+int srch_cmp(unsigned char c1, unsigned char c2)
 {
 	return casecmp(&c1, &c2, 1);
 }
@@ -524,9 +527,25 @@ struct link *get_last_link(struct f_data_c *f)
 	return l;
 }
 
+void fixup_select_state(struct form_control *fc, struct form_state *fs)
+{
+	int i;
+	if (fs->state >= 0 && fs->state < fc->nvalues && !strcmp(fc->values[fs->state], fs->value)) return;
+	for (i = 0; i < fc->nvalues; i++) {
+		if (!strcmp(fc->values[i], fs->value)) {
+			fs->state = i;
+			return;
+		}
+	}
+	fs->state = 0;
+	if (fs->value) mem_free(fs->value);
+	if (fc->nvalues) fs->value = stracpy(fc->values[0]);
+	else fs->value = stracpy("");
+}
+
 void init_ctrl(struct form_control *form, struct form_state *fs)
 {
-	if (fs->value) mem_free(fs->value);
+	if (fs->value) mem_free(fs->value), fs->value = NULL;
 	switch (form->type) {
 		case FC_TEXT:
 		case FC_PASSWORD:
@@ -542,9 +561,13 @@ void init_ctrl(struct form_control *form, struct form_state *fs)
 			break;
 		case FC_CHECKBOX:
 		case FC_RADIO:
-		case FC_SELECT:
 			fs->state = form->default_state;
 			break;
+		case FC_SELECT:
+			fs->value = stracpy(form->default_value);
+			fs->state = form->default_state;
+			fixup_select_state(form, fs);
+			break;
 	}
 }
 
@@ -642,7 +665,8 @@ void draw_form_entry(struct terminal *t, struct f_data_c *f, struct link *l)
 				set_only_char(t, x, y, fs->state ? 'X' : ' ');
 			break;
 		case FC_SELECT:
-			s = form->labels[fs->state];
+			fixup_select_state(form, fs);
+			s = fs->state < form->nvalues ? form->labels[fs->state] : (unsigned char *)"";
 			sl = s ? strlen(s) : 0;
 			for (i = 0; i < l->n; i++) {
 				x = l->pos[i].x + xp - vx;
@@ -1096,11 +1120,11 @@ void get_succesful_controls(struct f_data_c *f, struct form_control *fc, struct
 					sub->value = stracpy(form->default_value);
 					break;
 				case FC_SELECT:
-					if (fs->state >= form->nvalues) fs->state = 0;
-					sub->value = stracpy(form->values[fs->state]);
+					fixup_select_state(form, fs);
+					sub->value = stracpy(fs->value);
 					break;
 				case FC_IMAGE:
-					add_to_strn(&sub->name, !fi ? ".x" : ".y");
+					add_to_strn(&sub->name, fi ? ".x" : ".y");
 					sub->value = stracpy("0");
 					break;
 				default:
@@ -1177,16 +1201,19 @@ void encode_controls(struct list_head *l, unsigned char **data, int *len,
 	foreach(sv, *l) {
 		unsigned char *p = sv->value;
 		struct document_options o;
+		memset(&o, 0, sizeof o);
 		o.plain = 1;
 		d_opt = &o;
-		if (sv->type == FC_TEXTAREA) p = encode_textarea(sv->value);
+		/*if (sv->type == FC_TEXTAREA) p = encode_textarea(sv->value);*/
 		if (lst) add_to_str(data, len, "&"); else lst = 1;
 		encode_string(sv->name, data, len);
 		add_to_str(data, len, "=");
-		p2 = convert_string(convert_table, p, strlen(p));
+		if (sv->type == FC_TEXT || sv->type == FC_PASSWORD || sv->type == FC_TEXTAREA)
+			p2 = convert_string(convert_table, p, strlen(p));
+		else p2 = stracpy(p);
 		encode_string(p2, data, len);
 		mem_free(p2);
-		if (sv->type == FC_TEXTAREA) mem_free(p);
+		/*if (sv->type == FC_TEXTAREA) mem_free(p);*/
 	}
 }
 
@@ -1210,7 +1237,7 @@ void encode_multipart(struct session *ses, struct list_head *l, unsigned char **
 		bnd:
 		add_to_str(data, len, "--");
 		if (!(nbound_ptrs & (ALLOC_GR-1))) {
-			if (!(nbp = mem_realloc(bound_ptrs, nbound_ptrs + ALLOC_GR))) goto xx;
+			if (!(nbp = mem_realloc(bound_ptrs, (nbound_ptrs + ALLOC_GR) * sizeof(int)))) goto xx;
 			bound_ptrs = nbp;
 		}
 		bound_ptrs[nbound_ptrs++] = *len;
@@ -1230,7 +1257,9 @@ void encode_multipart(struct session *ses, struct list_head *l, unsigned char **
 			struct document_options o;
 			o.plain = 1;
 			d_opt = &o;
-			p = convert_string(convert_table, sv->value, strlen(sv->value));
+			if (sv->type == FC_TEXT || sv->type == FC_PASSWORD || sv->type == FC_TEXTAREA)
+				p = convert_string(convert_table, sv->value, strlen(sv->value));
+			else p = stracpy(sv->value);
 			add_to_str(data, len, p);
 			mem_free(p);
 		} else {
@@ -1320,7 +1349,8 @@ unsigned char *get_form_url(struct session *ses, struct f_data_c *f, struct form
 				pos = stracpy(pos);
 				*poss = 0;
 			}
-			strcat(go, "?");
+			if (strchr(go, '?')) strcat(go, "&");
+			else strcat(go, "?");
 			strcat(go, data);
 			if (pos) strcat(go, pos), mem_free(pos);
 		}
@@ -1391,8 +1421,10 @@ int enter(struct session *ses, struct f_data_c *f, int a)
 		else {
 			struct form_control *fc;
 			foreach(fc, f->f_data->forms)
-				if (fc->form_num == link->form->form_num && fc->type == FC_RADIO && !xstrcmp(fc->name, link->form->name))
-					find_form_state(f, fc)->state = 0;
+				if (fc->form_num == link->form->form_num && fc->type == FC_RADIO && !xstrcmp(fc->name, link->form->name)) {
+					struct form_state *ffs = find_form_state(f, fc);
+					if (ffs) ffs->state = 0;
+				}
 			fs->state = 1;
 		}
 	} else if (link->type == L_SELECT) {
@@ -1427,7 +1459,15 @@ void selected_item(struct terminal *term, void *pitem, struct session *ses)
 	if (f->vs->current_link == -1) return;
 	l = &f->f_data->links[f->vs->current_link];
 	if (l->type != L_SELECT) return;
-	if ((fs = find_form_state(f, l->form))) fs->state = item;
+	if ((fs = find_form_state(f, l->form))) {
+		struct form_control *form= l->form;
+		if (item >= 0 && item < form->nvalues) {
+			fs->state = item;
+			if (fs->value) mem_free(fs->value);
+			fs->value = stracpy(form->values[item]);
+		}
+		fixup_select_state(form, fs);
+	}
 	draw_doc(ses->term, f, 1);
 	print_screen_status(ses);
 	redraw_from_window(ses->win);
@@ -1533,10 +1573,11 @@ int field_op(struct session *ses, struct f_data_c *f, struct link *l, struct eve
 						int y;
 						for (y = 0; ln[y].st; y++) if (fs->value + fs->state >= ln[y].st && fs->value + fs->state < ln[y].en + (ln[y+1].st != ln[y].en)) {
 							fs->state = ln[y].en - fs->value;
-							goto y;
+							if (fs->state && fs->state < strlen(fs->value) && ln[y+1].st == ln[y].en) fs->state--;
+							goto yyyy;
 						}
 						fs->state = strlen(fs->value);
-						y:
+						yyyy:
 						mem_free(ln);
 					}
 				} else fs->state = strlen(fs->value);
@@ -1551,6 +1592,7 @@ int field_op(struct session *ses, struct f_data_c *f, struct link *l, struct eve
 				break;
 			case ACT_PASTE_CLIPBOARD: {
 				char *clipboard = get_clipboard_text();
+				if (!clipboard) break;
 				if (!form->ro && strlen(clipboard) <= form->maxlength) {
 					unsigned char *v;
 					if (v = mem_realloc(fs->value, strlen(clipboard) +1)) {
@@ -1798,8 +1840,10 @@ int frame_ev(struct session *ses, struct f_data_c *fd, struct event *ev)
 			case ACT_UP: rep_ev(ses, fd, up, 0); break;
 			case ACT_COPY_CLIPBOARD: {
 				char *current_link = print_current_link(ses);
-				set_clipboard_text( current_link );
-				mem_free(current_link);
+				if (current_link) {
+					set_clipboard_text( current_link );
+					mem_free(current_link);
+				}
 				break;
 			}
 			case ACT_SCROLL_UP: rep_ev(ses, fd, scroll, -1 - !ses->kbdprefix.rep); break;
@@ -2311,7 +2355,7 @@ unsigned char *print_current_linkx(struct f_data_c *fd, struct terminal *term)
 	struct link *l;
 	unsigned char *m;
 	if (!fd) return NULL;
-	if (fd->vs->current_link == -1 || fd->f_data->frame) return NULL;
+	if (fd->vs->current_link == -1 || fd->vs->current_link >= fd->f_data->nlinks || fd->f_data->frame) return NULL;
 	l = &fd->f_data->links[fd->vs->current_link];
 	if (l->type == L_LINK) {
 		if (!l->where && l->where_img) {
